<?php

namespace Goldenscarab\Modulus\App\Http\Controllers\App;

use Illuminate\Support\Str;
use Illuminate\Http\Request;
use App\Http\Controllers\Controller;

use Nwidart\Modules\Facades\Module;
use Illuminate\Support\Facades\Cache;
use Illuminate\Support\Facades\Artisan;
use Illuminate\Support\Facades\Session;
use Illuminate\Support\Facades\Redirect;
use Illuminate\Support\Facades\Response;
use Goldenscarab\Modulus\App\Http\Repositories\ModuleRepository;
use Goldenscarab\Modulus\App\Http\Services\Permission\Facade\PermissionService;


/**
 * Controlleur de Modules
 *
 *
 * @author      Sylvain CARRE
 * @version     v1 Administration des streams à partir de modèles
 * @copyright   2018 Goldenscarab ©
 * @license     MIT

 */
class ModuleController extends Controller
{

    private $module_repo;

    /**
     * Constructeur du controller ModuleStreamController
     */
    public function __construct(ModuleRepository $module_repo)
    {
        $this->module_repo = $module_repo;

        // Définition des permissions par méthode
        $this->middleware('permission:module-read')->only(['index', 'search']);
        $this->middleware('permission:module-create')->only(['import']);
        $this->middleware('permission:module-update')->only(['install', 'uninstall', 'reset']);
        $this->middleware('permission:module-delete')->only(['delete']);

        view()->share('route_prefix', "modulus.module");
        view()->share('can_prefix', "module");
        view()->share('route_param', current_query_to_string());
    }

    /**
     * Méthode par défaut - Affichage de la liste des champs
     * @return View     Liste des champs
     */
    public function index()
    {
        // Récupération des champs personnalisé
        $modules = Module::toCollection();

        // Désactiver les modules non installé
        $modules->each(function(&$m, $key) {
            // Recherche de la config du module dans modulus
            $module = $this->module_repo->findBy('slug', $m->getAlias());

            if (is_null($module)) {
                $m->disable();
            }
        });

        // On supprime de la liste les modules caché
        $modules = $modules->filter(function($module, $key) {
            return $module->get('hidden') == false;
        });

        $breadcrumb = array(
            'Accueil' => route('modulus.dashboard'),
            'Modules' => null
        );

    	return view('modulus::app.module.index')->with([
            'seo_title'  => 'Module liste',
            'title'      => '<i class="fa fa-puzzle-piece mr-2"></i>Modules',
            'subtitle'   => 'Liste des modules disponibles',
            'breadcrumb' => $breadcrumb,
            'modules'    => $modules
    	]);
    }


    public function install($alias)
    {
        $module = Module::findOrFail($alias);


        // Le module n'est pas activé on recharge pour prendre en compte la config du module
        if ($module->isStatus(0)) {
            $module->enable();
            return Redirect::route('modulus.module.install', $alias);
        }

        // On recherche les autoloads
        //system('composer dump-autoload -d ' . base_path());
        Artisan::call('config:clear');

        // Migrations des tables du module et semence
        Artisan::call('module:migrate', ['module' => $module->getName()]);
        Artisan::call('module:seed', ['module' => $module->getName()]);

        // Configuration dans Modulus
        $module_config = array(
            'name'     => $module->getName(),
            'slug'     => $module->getAlias(),
            'menu'     => config($module->getAlias() . '.menu'),
            'priority' => $module->get('priority')
        );

        $this->module_repo->store(collect($module_config));

        // Ajout des permissions d'accès au module
        PermissionService::addPermissionsGroup('Module ' . $module->getName());

        Session::flash('success', "Module installé avec succès");

        return Redirect::route('modulus.module.index');
    }

    public function uninstall($alias)
    {
        // Récupération du module
        $module = Module::findOrFail($alias);
        // Suppression des migrations des tables du module
        Artisan::call('module:migrate-rollback', ['module' => $module->getName()]);

        // Suppression de la configuration back office
        $this->module_repo->destroy($module->getAlias());

        // Suppression des permissions d'accès au module
        PermissionService::deletePermissionsGroup('Module ' . $module->getName());

        // On désactive le module
        $module->disable();

        Artisan::call('config:clear');

        Session::flash('success', "Module désinstallé avec succès");

        return Redirect::route('modulus.module.index');
    }

    public function refresh($alias)
    {
        $module = Module::findOrFail($alias);

        //system('composer dump-autoload -d ' . base_path());
        Artisan::call('config:clear');

        // Suppression de la configuration back office
        $this->module_repo->destroy($module->getAlias());

        $module_config = array(
            'name'     => $module->getName(),
            'slug'     => $module->getAlias(),
            'menu'     => config($module->getAlias() . '.menu'),
            'priority' => $module->get('priority')
        );

        $this->module_repo->store(collect($module_config));

        Session::flash('success', "Module mis à jour avec succès");

        return Redirect::route('modulus.module.index');
    }

}
