<?php

namespace Goldenscarab\Modulus\App\Http\Controllers\App;

use App\Http\Controllers\Controller;
use Goldenscarab\Modulus\App\Models\Permission;
use Illuminate\Http\Request;

use Illuminate\Support\Facades\Session;
use Illuminate\Support\Facades\Redirect;
use Goldenscarab\Modulus\App\Http\Requests\PermissionStoreRequest;
use Goldenscarab\Modulus\App\Http\Repositories\PermissionRepository;


class PermissionController extends Controller
{
    private $permission_repo;

    public function __construct(PermissionRepository $permission)
    {
        $this->permission_repo = $permission;

        $this->middleware('permission:permission-read')->only('index');
        $this->middleware('permission:permission-create')->only(['create']);
        $this->middleware('permission:permission-update')->only(['update']);
        $this->middleware('permission:permission-delete')->only('delete');

        view()->share('route_prefix', "modulus.user.permission");
    }

    public function index(Request $request)
    {
        // Option de tri de colonne
        $sorting   = $request->get('sorting', 'id');
        $direction = $request->get('direction', 'asc');
        $perpage   = $request->get('perpage', 999);

        $search = $request->get('search');

        $permissions = $this->permission_repo->list($sorting, $direction, $perpage, $search);

        $permissions = $permissions->unique('short_name');

        $breadcrumb = array(
            'Accueil'      => route('modulus.dashboard'),
            'Utilisateurs' => route('modulus.user.index'),
            'Permissions'  => null
        );

    	return view('modulus::app.user.permission.list')->with([
            'title'       => '<i class="fa fa-unlock-alt mr-1" aria-hidden="true"></i>Permissions',
            'breadcrumb'  => $breadcrumb,
            'permissions' => $permissions,
    	]);
    }

    public function create()
    {
        $permission   = $this->permission_repo->new();

        $breadcrumb = array(
            'Accueil'      => route('modulus.dashboard'),
            'Utilisateurs' => route('modulus.user.index'),
            'Permissions'  => route('modulus.user.permission.index'),
            'Ajouter'      => null
        );

        return view('modulus::app.user.permission.form')->with([
            'title'      => '<i class="fa fa-unlock-alt mr-1" aria-hidden="true"></i>Permission',
            'subtitle'   => $permission->display_name,
            'action'     => '<i class="fa fa-plus mr-2"></i>Ajouter',
            'breadcrumb' => $breadcrumb,
            'item'       => $permission,
        ]);
    }

    public function update($id)
    {
        $permission   = $this->permission_repo->findOrFail($id);

        $breadcrumb = array(
            'Accueil'      => route('modulus.dashboard'),
            'Utilisateurs' => route('modulus.user.index'),
            'Permissions'  => route('modulus.user.permission.index'),
            'Modifier'      => null
        );

        return view('modulus::app.user.permission.form')->with([
            'title'        => '<i class="fa fa-unlock-alt mr-1" aria-hidden="true"></i>Permission',
            'subtitle'     => $permission->display_name,
            'action'       => '<i class="fa fa-pencil-square-o mr-2"></i>Modifier',
            'breadcrumb'   => $breadcrumb,
            'item'         => $permission,
        ]);
    }

    public function store(PermissionStoreRequest $request, $id = null)
    {
        $permission_data = collect($request->except(['_token']));


        if (is_null($id)) {
            $base_name = $request->get('name');
        } else {
            $permission = $this->permission_repo->find($id);
            $base_name = $permission->short_name;
        }

        // Création / mise à jour des enfants de la permission
        foreach (Permission::getSuffixes() as $suffix => $description) {
            $name = $base_name . '-' . $suffix;
            $perm = $this->permission_repo->findBy('name', $name);

            $this->permission_repo->store(collect([
                'name'         => $request->get('name') . '-' . $suffix,
                'display_name' => $request->get('display_name') . ' ' . ucfirst($suffix),
                'description'  => $request->get('description') . ' ' . $description,
            ]), data_get($perm, 'id'));
        }

        Session::flash('success', "Permission ". (is_null($id) ? 'créée' : 'modifiée') ." avec succès");

        return Redirect::route('modulus.user.permission.index');
    }

    public function delete($id)
    {
        $permission = $this->permission_repo->findOrFail($id);
        $base_name = $permission->short_name;

        foreach (Permission::getSuffixes() as $suffix => $description) {
            $name = $base_name . '-' . $suffix;
            $perm = $this->permission_repo->findBy('name', $name);
            $perm->delete();
        }

        Session::flash('success', 'Permission supprimée avec succès');

        return Redirect::route('modulus.user.permission.index');
    }

}
