<?php

namespace Goldenscarab\Modulus\App\Http\Controllers\App;

use App\Http\Controllers\Controller;
use Illuminate\Http\Request;

use Goldenscarab\Modulus\App\Http\Requests\RoleStoreRequest;
use Illuminate\Support\Facades\Session;
use Illuminate\Support\Facades\Redirect;
use Goldenscarab\Modulus\App\Http\Repositories\RoleRepository;
use Goldenscarab\Modulus\App\Http\Repositories\PermissionRepository;


class RoleController extends Controller
{

    private $role_repo;

    private $permission_repo;

    public function __construct(RoleRepository $role)
    {
        $this->role_repo = $role;
        $this->permission_repo = new PermissionRepository;

        $this->middleware('permission:role-read')->only('index');
        $this->middleware('permission:role-create')->only(['create']);
        $this->middleware('permission:role-update')->only(['update']);
        $this->middleware('permission:role-delete')->only('delete');

        view()->share('route_prefix', "modulus.user.role");
    }

    public function index(Request $request)
    {
        // Option de tri de colonne
        $sorting   = $request->get('sorting', 'id');
        $direction = $request->get('direction', 'asc');
        $perpage   = $request->get('perpage', 10);

        $search = $request->get('search');

        $roles = $this->role_repo->list($sorting, $direction, $perpage, $search);

        $breadcrumb = array(
            'Accueil'      => route('modulus.dashboard'),
            'Utilisateurs' => route('modulus.user.index'),
            'Rôles'        => null
        );

    	return view('modulus::app.user.role.list')->with([
			'title'        => '<i class="fa fa-tags mr-1" aria-hidden="true"></i>Rôles',
            'breadcrumb'   => $breadcrumb,
			'roles'        => $roles,
    	]);
    }

    public function create()
    {
        $role  = $this->role_repo->new();

        $perms = $this->permission_repo->all();

        $permissions = $this->formatPermissions($perms);

        $breadcrumb = array(
            'Accueil'      => route('modulus.dashboard'),
            'Utilisateurs' => route('modulus.user.index'),
            'Rôles'        => route('modulus.user.role.index'),
            'Ajouter'      => null
        );

        return view('modulus::app.user.role.form')->with([
            'title'        => '<i class="fa fa-tags mr-1" aria-hidden="true"></i>Rôle',
            'action'       => '<i class="fa fa-plus mr-2"></i>Ajouter',
            'breadcrumb'   => $breadcrumb,
            'item'         => $role,
            'permissions'  => $permissions
        ]);
    }

    public function update($id)
    {
        $role  = $this->role_repo->findOrFail($id);

        $perms = $this->permission_repo->all();

        $permissions = $this->formatPermissions($perms);

        $breadcrumb = array(
            'Accueil'      => route('modulus.dashboard'),
            'Utilisateurs' => route('modulus.user.index'),
            'Rôles'        => route('modulus.user.role.index'),
            'Modifier'     => null
        );

        return view('modulus::app.user.role.form')->with([
            'title'        => '<i class="fa fa-tags mr-1" aria-hidden="true"></i>Rôle',
            'subtitle'     => $role->display_name,
            'action'       => '<i class="fa fa-pencil-square-o mr-2"></i>Modifier',
            'breadcrumb'   => $breadcrumb,
            'item'         => $role,
            'permissions'  => $permissions
        ]);
    }

    private function formatPermissions($perms)
    {
         $permissions = [];

        // Traitement sur les permissions
        $perms->each(function($item, $key) use (&$permissions) {

            // Si la permission est conforme
            if (preg_match('/-(.[^-]+)$/i', $item->name, $match)) {

                $group = str_replace($match[0], '', $item->name);
                $perm  = $match[1];
            } else {
                $group = $item->name;
                $perm  = 'other';
            }

            $name = trim(str_ireplace($perm, '', $item->display_name));

            $permissions[$name][$perm] = $item->id;

        });

        return $permissions;
    }

    public function store(RoleStoreRequest $request, $id = null)
    {
        $role_data = collect($request->except(['_token']));

        $this->role_repo->store($role_data, $id);

        Session::flash('success', "Rôle ". (is_null($id) ? 'crée' : 'modifié') ." avec succès");

        return Redirect::route('modulus.user.role.index');
    }

    public function delete($id)
    {
        $this->role_repo->destroy($id);

        Session::flash('success', 'Rôle supprimé avec succès');

        return Redirect::route('modulus.user.role.index');
    }

}
