<?php

namespace Goldenscarab\Modulus\App\Http\Controllers\App;

use App\Http\Controllers\Controller;
use Illuminate\Support\Str;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\DB;
use Illuminate\Support\Facades\Hash;
use Illuminate\Support\Facades\Mail;
use Maatwebsite\Excel\Facades\Excel;
use Goldenscarab\Modulus\App\Mail\SendActiveAccountRequest;
use Goldenscarab\Modulus\App\Http\Requests\UserStoreRequest;
use Illuminate\Support\Facades\Session;
use Illuminate\Support\Facades\Redirect;
use Goldenscarab\Modulus\App\Http\Repositories\RoleRepository;
use Goldenscarab\Modulus\App\Http\Repositories\UserRepository;
use Goldenscarab\Modulus\App\Http\Requests\ProfileStoreRequest;


class UserController extends Controller
{

    private $user_repo;
    private $role_repo;

    public function __construct(UserRepository $user)
    {
        $this->user_repo = $user;
        $this->role_repo = new RoleRepository;

        // Définition des permissions par méthode
        $this->middleware('permission:user-read')->only('index');
        $this->middleware('permission:user-create')->only('create');
        $this->middleware('permission:user-update')->only('update');
        $this->middleware('permission:user-delete')->only('delete');

        view()->share('route_prefix', "modulus.user");
    }

    public function index(Request $request)
    {
        // Option de tri de colonne
        $sorting   = $request->get('sorting', 'id');
        $direction = $request->get('direction', 'asc');
        $perpage   = $request->get('perpage', 10);

        // Option de Recherche
        $search = $request->get('search');
        $role   = $request->get('role');

        // Récupère des listes
        $users = $this->user_repo->list($sorting, $direction, $perpage, $search, $role);
        $roles = $this->role_repo->all();

        $breadcrumb = array(
            'Accueil'      => route('modulus.dashboard'),
            'Utilisateurs' => null,
        );

        return view('modulus::app.user.list')->with([
            'title'        => '<i class="fa fa-users mr-1" aria-hidden="true"></i>Utilisateurs',
            'breadcrumb'   => $breadcrumb,
            'users'        => $users,
            'roles'        => $roles,
        ]);
    }

    public function create()
    {
        $user   = $this->user_repo->new();
        $avatars = $this->user_repo->getAvatars();
        $roles  = $this->role_repo->all();

        $breadcrumb = array(
            'Accueil'      => route('modulus.dashboard'),
            'Utilisateurs' => route('modulus.user.index'),
            'Ajouter'      => null
        );

        return view('modulus::app.user.form')->with([
            'title'      => '<i class="fa fa-users mr-1" aria-hidden="true"></i>Ajouter un utilisateur',
            'action'     => '<i class="fa fa-plus mr-2"></i>Ajouter',
            'breadcrumb' => $breadcrumb,
            'item'       => $user,
            'avatars'    => $avatars,
            'roles'      => $roles
        ]);
    }

    public function update($id)
    {
        $user   = $this->user_repo->findOrfail($id);
        $avatars = $this->user_repo->getAvatars();
        $roles  = $this->role_repo->all();

        $breadcrumb = array(
            'Accueil'      => route('modulus.dashboard'),
            'Utilisateurs' => route('modulus.user.index'),
            'Modifier'     => null
        );

        return view('modulus::app.user.form')->with([
            'title'      => '<i class="fa fa-users mr-1" aria-hidden="true"></i>Modifier un utilisateur',
            'action'     => '<i class="fa fa-pencil-square-o mr-2"></i>Modifier',
            'breadcrumb' => $breadcrumb,
            'item'       => $user,
            'avatars'    => $avatars,
            'roles'      => $roles
        ]);
    }

    public function store(UserStoreRequest $request, $id = null)
    {
        // On récupère les données utilisateur
        $user_data = collect($request->except(['_token']));

        // On sauvegarde les données utilisateur
        $user = $this->user_repo->store($user_data, $id);

        // Cas d'une création
        if (is_null($id)) {

            $this->passwordReset($user);

            Session::flash('success', "Utilisateur ajouté avec succès");

            return Redirect::route('modulus.user.index');
        }

        Session::flash('success', "Utilisateur modifié avec succès");

        return Redirect::route('modulus.user.index', $request->query());
    }

    /**
     * Export des utilisateurs
     * @return XLS Fichier Excel
     */
    public function export(Request $request)
    {
        // Option de tri de colonne
        $sorting   = $request->get('sorting', 'id');
        $direction = $request->get('direction', 'desc');

        // Option de Recherche
        $search = $request->get('search');
        $role   = $request->get('role');
        $sector = $request->get('sector');
        $job    = $request->get('job');

        // Récupère des listes
        $users = $this->user_repo->list($sorting, $direction, 9999, $search, $role, $sector, $job);

        //Création d'un nom de fichier
        $file_name = date('Ymd_His_') . 'export-users';

        // Construction du tableau d'export
        $datas_export = [];

        // Parcours des données
        foreach ($users as $user) {

            $entrie = [
                'ID'               => $user->id,
                'Nom'              => $user->lastname,
                'Prénom'           => $user->firstname,
                'E-mail'           => $user->email,
                'Société'          => $user->company,
                'Activité'         => $user->activity,
                'Téléphone'        => $user->telephone,
                'Adresse'          => $user->address,
                'Code postal'      => $user->zipcode,
                'Ville'            => $user->city,
                'Commentaire'      => $user->comment,
                'Email validé'     => $user->validated ? 'Oui' : 'Non',
                'Rôle'             => $user->roles->isEmpty() ? '' : $user->roles->first()->display_name,
                'Actif'            => $user->active ? 'Oui' : 'Non',
                'Mis à jour le'    => $user->updated_at->format('d/m/Y H:i:s'),
                'Créé le'          => $user->created_at->format('d/m/Y H:i:s')
            ];

            // On ajout la ligne (entrée data) au tableau d'export
            $datas_export[] = $entrie;
        }

        // Création du document Excel
        Excel::create($file_name, function($excel) use ($datas_export) {

            // Paramétrage du tableau Excel
            $excel->setTitle("Export du ". date('d/m/Y H:i:s'));
            $excel->setCreator('BMV Communication');
            $excel->setDescription('');

            // Création d'une nouvelle feuille de calcul
            $excel->sheet('Utilisateur', function($sheet) use ($datas_export) {

                // Mise en style de la feuille de calcul
                $sheet->setFontFamily('Arial');
                $sheet->setFontSize(10);

                // Ajout des données
                $sheet->fromArray($datas_export, null, 'A1');

                // Ajout d'une ligne d'entête
                $sheet->prependRow(1, ["Export du ". date('d/m/Y H:i:s')]);
                $sheet->prependRow(2, [""]);

                // Mise à caractère gras des titres (ligne 3)
                $sheet->row(3, function($row) {
                    $row->setFontWeight('bold');
                });

                $sheet->setFreeze('A4');
            });
        })->download('xls');

        exit;
    }

    public function profil()
    {
        $item = auth()->user();

        return view('modulus::app.user.profil')->with([
            'title'    => '<i class="fa fa-user mr-1"></i>Profil utilisateur',
            'item'     => $item,
        ]);
    }

    public function profilPost(ProfileStoreRequest $request)
    {
        $data = collect($request->except(['_token']));
        $user = $this->user_repo->store($data, auth()->user()->id);

        Session::flash('success', "Profil enregistré avec succès");

        return Redirect::back();
    }

    public function delete($id)
    {
        $this->user_repo->destroy($id);

        Session::flash('success', 'Utilisateur supprimé avec succès');

        return Redirect::route('modulus.user.index');
    }

    public function sendCreatedAccountMail($id)
    {
        $user = $this->user_repo->findOrFail($id);

        $this->passwordReset($user);

        Session::flash('success', 'Mail de création de compte envoyé avec succès');

        return Redirect::route('modulus.user.index');
    }

    private function passwordReset($user)
    {
        // On génère une nouvelle clée
        $reset_token = hash_hmac('sha256', Str::random(40), config('app.key'));

        // Ajoute une entrée de regénération de mot de passe
        DB::table('password_resets')->insert([
            'email'      => $user->email,
            'token'      => Hash::make($reset_token),
            'created_at' => \Carbon\Carbon::now(),
        ]);

        // On envoi le mail de notification d'activation de compte
        Mail::to($user->email)->queue(new SendActiveAccountRequest($user, $reset_token));
    }

}
