<?php
	namespace Goldenscarab\Modulus\App\Http\Services\LogTracker;

	use Illuminate\Support\Collection;
	use \Carbon\Carbon;


	/*
		 _               _____               _
		| |    ___   __ |_   _| __ __ _  ___| | _____ _ __
		| |   / _ \ / _` || || '__/ _` |/ __| |/ / _ \ '__|
		| |__| (_) | (_| || || | | (_| | (__|   <  __/ |
		|_____\___/ \__, ||_||_|  \__,_|\___|_|\_\___|_|
		            |___/
	 */

	/**
	 *
	 * Loggeur de trace
	 *
	 * @category   Library
	 * @package    LogTracker.php
	 * @author     Sylvain CARRE
	 * @copyright  2017 Goldenscarab
	 * @version    v1.0
	 * @license    MIT
	 *
	 */


	class LogTrackerService
	{
		/* Attributs */
		private $current_month_log_path_name;

		private $separator = ':;:';


		/* Méthodes */
		public function __construct()
		{
			$this->current_month_log_path_name = 'logs/' . date('Y-m') . '_tracker.log';
		}

		/**
		 * Ajout d'une ligne au fichier de log avec les informations
		 * liées à l'ip utilisateur
		 * @param  Collection $tracker Les données utilisateur
		 * @return null
		 */
		public function save(Collection $tracker)
		{
			$data = array(
                'date'            => date('Y-m-d H:i:s'),
                'ip'              => $tracker->get('ip'),
                'user_email'      => $tracker->get('user_email'),
                'request_uri'     => $tracker->get('request_uri'),
                'request_method'  => $tracker->get('request_method'),
                'http_user_agent' => $tracker->get('http_user_agent'),
                'provider'        => $tracker->get('provider'),
                'zip_code'        => $tracker->get('zip_code'),
                'city'            => $tracker->get('city'),
                'region'          => $tracker->get('region'),
                'country_name'    => $tracker->get('country_name'),
                'latitude'        => $tracker->get('latitude'),
                'longitude'       => $tracker->get('longitude'),
			);

			// Sauvegarde des données dans le fichier .log avec un séparateur de fin ','
           \Storage::prepend($this->current_month_log_path_name, implode($this->separator, $data));
		}

		/**
		 * Récupère les logs du mois courant
		 * @return Collection Une collection des logs
		 */
		public function getCurrentMonth()
		{
			// On récupère les logs du fichier
			$logs = $this->getLogsContent($this->current_month_log_path_name);

			// On ajoute les logs du mois dans les logs de l'année
			return $logs;
		}

		/**
		 * Récupère les logs du mois dernier
		 * @return Collection Une collection des logs
		 */
		public function getLastMonth()
		{
			// Création de la date du mois dernier
			$last_month_date = Carbon::now()->subMonth()->format('Y-m');

			// Création du nom de fichier log
			$last_month_log_path_name = 'logs/' . $last_month_date . '_tracker.log';

			// On récupère les logs du fichier
			$logs = $this->getLogsContent($last_month_log_path_name);

			return $logs;

		}

		/**
		 * Récupère les dernières 200 IP qui ont visitées l'application
		 * @return Collection Une collection des logs
		 */
		public function getLastIP()
		{
			// On récupère les logs du fichier
			$logs = $this->getLogsContent($this->current_month_log_path_name);

			$log_by_ip = $logs->groupBy('ip')->take(50);

			unset($logs);

			$ips = collect();

			// Mise en forme des logs pour affichage
			foreach ($log_by_ip as $ip => $logs) {

				// On récupère le dernier log de l'ip
				$log = $logs->first();

				// Création des données liées à l'ip
				$data = new \StdClass;
				$data->ip              = $ip;
				$data->user            = $log->user_email ? $log->user_email : 'Invité';
				$data->request_uri     = $log->request_uri;
				$data->request_method  = $log->request_method;
				$data->provider        = $log->provider;
				$data->zip_code        = $log->zip_code;
				$data->city            = $log->city;
				$data->region          = $log->region;
				$data->country         = $log->country_name;
				$data->latitude        = $log->latitude;
				$data->longitude       = $log->longitude;
				$data->http_user_agent = $log->http_user_agent;
				$data->pages_view      = count($logs);
				$data->last_activity   = Carbon::createFromFormat('Y-m-d H:i:s', $log->date)->diffForHumans();


				$ips->put($ip, $data);

			}

			// On ajoute les logs du mois dans les logs de l'année
			return $ips;
		}

		/**
		 * Récupère les logs de l'année en cours
		 * @return Collection Une collection des logs
		 */
		public function getCurrentYear()
		{
			$logs_of_the_year = collect();
			$current_year     = date('Y');

			// Parcours des mois de l'année en cours
			for ($month = 1 ; $month <= date('m') ; $month++) {

				// Formatage du mois de l'année
				$month = sprintf('%02d', $month);

				// Création du chemin vers le log
				$path_file = 'logs/' . $current_year . '-' . $month . '_tracker.log';

				// On récupère les logs du fichier si il existe
				$logs = $this->getLogsContent($path_file);

				// On ajoute les logs du mois avec son ondice dans le tableau année
				$logs_of_the_year->put($month, $logs);

				// On libère de la mémoire
				unset($logs);

			}

			return $logs_of_the_year;
		}

		/**
		 * Récupère les logs de l'année dernière
		 * @return Collection Une collection des logs
		 */
		public function getLastYear()
		{
			$logs_of_last_year = collect();
			$last_year         = Carbon::now()->subYear()->format('Y');

			// Parcours des mois de l'année complète
			for ($month = 1 ; $month <= 12 ; $month++) {

				// Formatage du mois de l'année
				$month = sprintf('%02d', $month);

				// Création du chemin vers le log
				$path_file = 'logs/' . $last_year . '-' . $month . '_tracker.log';

				// On récupère les logs du fichier si il existe
				$logs = $this->getLogsContent($path_file);

				// On ajoute les logs du mois avec son ondice dans le tableau année
				$logs_of_last_year->put($month, $logs);

				// On libère de la mémoire
				unset($logs);
			}

			return $logs_of_last_year;
		}

		/**
		 * Lire un fichier de log depuis son chemin
		 * @param  String $path_file Le nom avec le chemin vers le fichier
		 * @return Collection            Les logs du fichier
		 */
		private function getLogsContent($path_file)
		{
			$logs = collect();
			// Si le fichier log existe
			if (\Storage::exists($path_file)) {

				$logs_temp = \Storage::get($path_file);

	        	$cols = array(
	                'date',
	                'ip',
	                'user_email',
	                'request_uri',
	                'request_method',
	                'http_user_agent',
	                'provider',
	                'zip_code',
	                'city',
	                'region',
	                'country_name',
	                'latitude',
	                'longitude',
				);

	   			$log_array = $this->getCsvToObject($logs_temp, $cols);

				// Création d'un collection
				$logs = collect($log_array);

				// On libère de la mémoire
				unset($log_array, $logs_temp);
			}

			return $logs;
		}


		public function getCsvToObject($datas, $keys = false)
		{
			$tabDataFormat = array(); //Tableau des datas formatées

			$tabDataBrut = explode("\n", $datas);

			foreach ($tabDataBrut as $line) {
				$datas_line = explode($this->separator, $line);

				$tabDataFormat[] = (object) array_combine($keys, $datas_line);
			}
			return $tabDataFormat;
		}



		/**
		 * Supprime de dossier logs contenant les logs de l'application
		 * @return Boolean Vrai si le dossier est supprimé, Faux s'il n'est pas trouvé
		 */
		public static function purge()
		{
			return \Storage::deleteDirectory('./logs');
		}
	}

