<?php

namespace Goldenscarab\Modulus\App\Http\Services\Parser;

use Goldenscarab\Modulus\App\Http\Services\App\Facade\AppService;
use Goldenscarab\Modulus\App\Http\Services\Parser\Workers\ParserFactory;


class Parser
{
	/**
	 * Liste des tags trouvé dans le contenu à traiter
	 * @var Array
	 */
	private $tags;

	/**
	 * Tableau associatif entre le tag et son rendu
	 * @var Array
	 */
	private $correlation_table;

	/**
	 * Regex permettant de trouver les tags
	 * @var [type]
	 */
	private $tag_regex = null;



	public function __construct()
	{
		// Recherche des tags du type : [ mon-tag ]
		$this->tag_regex = "/\[[\s]([^\]\s]*)[\s]\]/";
	}


	/**
	 * Traite des données html en recherchant et remplacant les tags trouvés
	 *    1 - Récupération de la liste des tags présent dans le code
	 *    2 - Analyse de chaque tags
	 *    3 - Recherche du worker adapté au tag
	 *    4 - Traitement du contenu à remplacer
	 *    5 - Remplacement des tags par leur contenu
	 *    6 - Remplacement des tags de non rendu
	 * @param  String $html Contenu html
	 * @return String       Contenu traité
	 */
	public function render($html)
	{
		// Niveau de recherche des TAG
		$search_level = 0;

		// Traitement des tags présents dans le html.
		// On réitère l'opération tant qu'il y a des tags
		do {

			$search_level++;

			// Récupération des tags présent dans le code
			$this->tags = $this->getTagsFromHtml($html, $this->tag_regex);

			// Transmission de la liste des tags à traité à la fabrique
			// La fabrique doit renvoyer un tableau associatif avec le rendu de chaque tag
			// Il s'agit de la table de correlation ([tag] => "rendu html")
			$this->correlation_table = ParserFactory::make($this->tags);

			// Remplacement des tags dans le html, grâce à la table de correlation
			$this->tagsRendering($this->correlation_table, $html);

		} while (count($this->tags[0]) > 0 && $search_level < 10);

		//\AppService::logLoad("Parser Access");

		// On traite potentiellement le code blade resultant
		$html = AppService::bladeRender($html);

		$this->replaceTagNoRender($html);

		return $html;
	}


	/**
	 * Récupère les tags présent dans le code à partir d'une expression régulière
	 * @param  String $html  Le Html à traiter
	 * @param  String $regex La regex permettant la d'extraire des tags du code
	 * @return Array        Tableau des tags présent dans le code
	 */
	private function getTagsFromHtml($html, $regex)
	{
		$tags = null;

		// Récupération des différents tags présent dans le contenu html
		preg_match_all($regex, $html, $tags);

		// On renvoi uniquement les tags sans leur carcasse ( [] )
		return $tags;
	}

	/**
	 * Remplace les tags dans le html par le rendu associé
	 * @param  array $correlation_table  La table de correlation [tag => rendu]
	 * @param  string $html              Le HTML à traiter
	 * @return void                      Le traitement est fait par référence
	 */
	private function tagsRendering($correlation_table, &$html)
	{

		// Avant tout on vérifie qu'un tableau à bien été fourni
		if (is_array($correlation_table)) {

			// On parcours la tableau de correlation pour remplacement
			foreach ($correlation_table as $tag => $render) {

				// Remplacement du tag par son rendu
				$html = str_replace($tag, $render, $html);
			}

		}
	}

	/**
	 * Remplace les tags ne devant pas être traité par le parseur
	 * @param  string &$html Le HTML à traiter
	 * @return void          Le traitement est fait par référence
	 */
	private function replaceTagNoRender(&$html)
	{
		$html = str_replace('{#', '{{', $html);
        $html = str_replace('#}', '}}', $html);
        $html = str_replace('{##', '{!!', $html);
		$html = str_replace('##}', '!!}', $html);
		$html = str_replace('{@}', '@', $html);

		$html = str_replace('<#php', '<?php', $html);
		$html = str_replace('#>', '?>', $html);
	}
}
